#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include <stdlib.h>

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1
#define RQ_ON    	  2
#define RQ_OFF    	  3

typedef struct HidReport
{

    uint8_t button0:1;
    uint8_t button1:1;
    uint8_t button2:1;
    uint8_t unused:5;

} HidReport_t;

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uchar idleRate; // Unused

static HidReport_t reportBuffer;

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] =
{

    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x05,                    // USAGE (Game Pad)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x03,                    //   USAGE_MAXIMUM (Button 8)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};


int state_random=0,randomed=3,x=-1,y=-1,z=-1;

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

uint16_t diff(uint8_t channel){
    int x1 = read_adc(channel);
    int x2 = read_adc(channel);
    int value =0;
    if(x1 >= x2){
        value = x1-x2;
    }else{
        value = x2-x1;
    }

    if(value > 10)
        return 0;
    else
        return 1;
}
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    //static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    //static uint16_t result,light;

/*
    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];

        if (led_val)
            PORTC |= (1<<led_no);
        else
            PORTC &= ~(1<<led_no);

        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
            switch_state = 1;
        else
            switch_state = 0;
        usbMsgPtr = (uchar*) &switch_state;

        return sizeof(switch_state);
    }

	else if(rq->bRequest == RQ_ON)
	{
		if(x==y&&y==z&&z!=-1)
		{
			randomed=1;
			usbMsgPtr=(void*)&randomed;
			return sizeof(randomed);
		}
		else if(z!=-1)
		{
			randomed=0;
			usbMsgPtr=(void*)&randomed;
			return sizeof(randomed);
		}
	}

	else if(rq->bRequest == RQ_OFF)
	{
		state_random=0;
		return 0;
	}
	*/

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }


    return 0;   /* default for not implemented requests: return no data back to host */
}

void buildReport()
{

		if(randomed==1)
		{
			reportBuffer.button0 = randomed;
			reportBuffer.button1 = 0;
			reportBuffer.button2 = 0;
		}
		else if(randomed==0)
		{
			reportBuffer.button0 = 0;
			reportBuffer.button1 = 1;
			reportBuffer.button2 = 0;
		}
		else if (randomed== 2)
		{
			reportBuffer.button0 = 0;
			reportBuffer.button1 = 0;
			reportBuffer.button2 = 0;
		}
		else if(randomed==3)
		{
			reportBuffer.button0 = 0;
			reportBuffer.button1 = 0;
			reportBuffer.button2 = 1;
		}
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    //DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = (1 << PC3);                 // Enable pull-up at PC3

	int ms=0,push=0;
	uint8_t num[11][2];
	DDRC  = 0b00011111;
	PORTC = 0b00100000;
	DDRB  = 0b00110111;
	PORTB = 0b00011101;
	num[0][0] = 0b00000100;
	num[0][1] = 0b00000000;
	num[1][0] = 0b00011111;
	num[1][1] = 0b00000000;
	num[2][0] = 0b00010000;
	num[2][1] = 0b00100000;
	num[3][0] = 0b00010010;
	num[3][1] = 0b00000000;
	num[4][0] = 0b00001011;
	num[4][1] = 0b00000000;
	num[5][0] = 0b00100010;
	num[5][1] = 0b00000010;
	num[6][0] = 0b00100000;
	num[6][1] = 0b00000010;
	num[7][0] = 0b00011110;
	num[7][1] = 0b00000000;
	num[8][0] = 0b00000000;
	num[8][1] = 0b00000000;
	num[9][0] = 0b00000010;
	num[9][1] = 0b00000000;
	num[10][0]= 0b00111011;
	num[10][1]= 0b00111101;
	uint8_t a = num[0][0];
	uint8_t A = 0b00011000;
	uint8_t b = num[1][0];
	uint8_t B = 0b00001100;
	uint8_t c = num[2][0];
	uint8_t C = 0b00001001;

	usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
    	buildReport();
			//if((PINB & 0b00000010)==0&&push==0)
			if(diff(PC5)==0&&push==0)
				push=1;
			if(push==1)
			{
				a=num[x][0];
				A=0b00010000|num[x][1];
				b=num[y][0];
				B=0b00000100|num[y][1];
				c=num[z][0];
				C=0b00000001|num[z][1];
				PORTB = A;
				PORTC = a;
				_delay_ms(1);
				PORTB = B;
				PORTC = b;
				_delay_ms(1);
		        PORTB = C;
		        PORTC = c;
				_delay_ms(1);
				randomed=2;
				if(ms > 5500) {ms =0; push=0;}
				else if(ms>4200);
	            else if(ms>=4000&&ms<=4200)
		        {
					if(x==y&&y==z)
						randomed=1;
					else if((x+1==y&&y+1==z)||(x-1==y&&y-1==z))
						randomed=3;
					else
						randomed=0;
		        }
		        else if(ms>3000&&ms<4000)
		        {
					if(ms%100==0){
		                z=rand()%10;}
		      	}
 				else if(ms>2000)
	 			{
					if(ms%50==0){
		               	z=rand()%10;}
					if(ms%100==0)
	            		y=rand()%10;
		    	}
		      	else if(ms>1000)
	      		{
					if(ms%50==0){
	             	y=rand()%10;
	              	z=rand()%10;}
					if(ms%100==0)x=rand()%10;
				}
				else //if(ms<=10000)
				{
					if(ms%50==0){
					x=rand()%10;
					y=rand()%10;
					z=rand()%10;}
				}
				ms+=1;
			}
			else //if((PINB & 0b00001000)==1)
			{
				//push=0;
				PORTB = 0b00111111;
				PORTC = 0b00111011;
				ms=0;
			}
			if(ms<4000 || ms>4200)
			randomed=2;
        usbPoll();

        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
